@@ -2,7 +2,7 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2007-2016 by Dominique Dumont.
+# This software is Copyright (c) 2007-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -119,14 +119,11 @@
             config_dir  => 'conf_data',
             auto_create => 1,
         },
-        write_config => [
-            {
-                backend    => 'cds_file',
-                config_dir => 'conf_data',
-                file       => 'mymaster.cds'
-            },
-            { backend => 'perl_file', config_dir => 'conf_data' }
-        ],
+        write_config => [{
+            backend    => 'cds_file',
+            config_dir => 'conf_data',
+            file       => 'mymaster.cds',
+        }],
 
         element => [
             std_id => {
