@@ -1,16 +1,16 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2016 by Dominique Dumont.
+# This software is Copyright (c) 2007-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Itself ;
-$Config::Model::Itself::VERSION = '2.003';
+$Config::Model::Itself::VERSION = '2.013';
 use Mouse ;
-use Config::Model 2.076;
+use Config::Model 2.114;
 use 5.010;
 
 use IO::File ;
@@ -21,7 +21,7 @@ use File::Find ;
 use File::Path ;
 use File::Basename ;
 use Data::Compare ;
-use Path::Tiny;
+use Path::Tiny 0.062;
 use Mouse::Util::TypeConstraints;
 
 my $logger = Log::Log4perl::get_logger("Backend::Itself");
@@ -259,7 +259,8 @@ sub read_all {
     my @files ;
     my $wanted = sub {
         push @files, $_ if ( $_->is_file and /\.pl$/
-                            and m!$read_dir/$root_model_file\b!
+                                 and m!$read_dir/$root_model_file\b!
+                                 and not m!\.d/!
                            ) ;
     } ;
     $read_dir->visit($wanted, { recurse => 1} ) ;
@@ -277,10 +278,12 @@ sub read_all {
         # now apply some translation to read model
         # - translate legacy warp parameters
         # - expand elements name
-        my $tmp_model = Config::Model -> new( skip_include => 1, legacy => $legacy ) ;
+        my @legacy = $legacy ? ( legacy => $legacy ) : () ;
+        my $tmp_model = Config::Model -> new( skip_include => 1, @legacy ) ;
+
         # @models order is important to write configuration class back in the same
         # order as the declaration
-        my @models = $tmp_model -> load ( 'Tmp' , $file ) ;
+        my @models = $tmp_model -> load ( 'Tmp' , $file->absolute ) ;
         push @all_models, @models;
 
         my $rel_file = $file ;
@@ -294,6 +297,8 @@ sub read_all {
             my $raw_model =  $tmp_model -> get_raw_model( $model_name ) ;
             my $new_model =  $tmp_model -> get_model( $model_name ) ;
 
+            $self->upgrade_model($model_name, $new_model);
+
             # track read class to identify later classes added by user
             $self->add_tracked_class($model_name);
 
@@ -374,6 +379,38 @@ sub read_all {
     return $self->{map} = \%class_file_map ;
 }
 
+# can be removed end of 2019 (after buster is released)
+sub upgrade_model {
+    my ($self, $config_class_name, $model) = @_ ;
+
+    my $multi_backend = 0;
+    foreach my $config (qw/read_config write_config/) {
+        my $ref = $model->{$config};
+        if ($ref and ref($ref) eq 'ARRAY') {
+            if (@$ref == 1) {
+                $model->{$config} = $ref->[0];
+            }
+            elsif (@$ref > 1){
+                $logger->warn("$config_class_name $config: cannot migrate multiple backends to rw_config");
+                $multi_backend++;
+            }
+        }
+    }
+
+    if ($model->{read_config} and not $multi_backend) {
+        say ("Model $config_class_name: moving read_config specification to rw_config");
+        $model->{rw_config} = delete $model->{read_config};
+    }
+
+    if ($model->{write_config} and not $multi_backend) {
+        say "Model $config_class_name: merging write_config specification in rw_config";
+        if (not $multi_backend) {
+            map {$model->{rw_config}{$_} = $model->{write_config}{$_} } keys %{$model->{write_config}} ;
+            delete $model->{write_config};
+        }
+    }
+}
+
 # internal
 sub get_perl_data_model{
     my $self = shift ;
@@ -498,16 +535,16 @@ sub write_all {
     $self->meta_instance->clear_changes ;
 }
 
-sub write_model_snippet {
+sub write_model_plugin {
     my $self = shift ;
     my %args = @_ ;
-    my $snippet_dir = delete $args{snippet_dir}
-      || croak __PACKAGE__," write_model_snippet: undefined snippet_dir";
-    my $model_file = delete $args{model_file}
-      || croak __PACKAGE__," write_model_snippet: undefined model_file";
-    croak "write_model_snippet: unexpected parameters ",join(' ', keys %args) if %args ;
+    my $plugin_dir = delete $args{plugin_dir}
+      || croak __PACKAGE__," write_model_plugin: undefined plugin_dir";
+    my $plugin_name = delete $args{plugin_name}
+        || croak __PACKAGE__," write_model_plugin: undefined plugin_name";
+    croak "write_model_plugin: unexpected parameters ",join(' ', keys %args) if %args ;
 
-    my $model = $self->meta_root->dump_as_data ;
+    my $model = $self->meta_root->dump_as_data(mode => 'custom') ;
     # print (Dumper( $model)) ;
 
     my @raw_data = @{$model->{class} || []} ;
@@ -517,23 +554,23 @@ sub write_model_snippet {
 
         # does not distinguish between notes from underlying model or snipper notes ...
         my @notes = $self->meta_root->grab("class:$class")->dump_annotations_as_pod ;
-        my $class_dir = $class.'.d';
-        $class_dir =~ s!::!/!g;
-        write_model_file ("$snippet_dir/$class_dir/$model_file", [], \@notes, [ $data ]);
+        my $plugin_file = $class.'.pl';
+        $plugin_file =~ s!::!/!g;
+        write_model_file ("$plugin_dir/$plugin_name/$plugin_file", [], \@notes, [ $data ]);
     }
 
     $self->meta_instance->clear_changes ;
 }
 
-sub read_model_snippet {
+sub read_model_plugin {
     my $self = shift ;
     my %args = @_ ;
-    my $snippet_dir = delete $args{snippet_dir}
-      || croak __PACKAGE__," write_model_snippet: undefined snippet_dir";
-    my $model_file = delete $args{model_file}
-      || croak __PACKAGE__," read_model_snippet: undefined model_file";
+    my $plugin_dir = delete $args{plugin_dir}
+      || croak __PACKAGE__," write_model_plugin: undefined plugin_dir";
+    my $plugin_name = delete $args{plugin_name}
+      || croak __PACKAGE__," read_model_plugin: undefined plugin_name";
 
-    croak "read_model_snippet: unexpected parameters ",join(' ', keys %args) if %args ;
+    croak "read_model_plugin: unexpected parameters ",join(' ', keys %args) if %args ;
 
     my @files ;
     my $wanted = sub {
@@ -541,33 +578,35 @@ sub read_model_snippet {
         push @files, $n if (-f $_ and not /~$/
                             and $n !~ /CVS/
                             and $n !~ m!.(svn|orig|pod)$!
-                            and $n =~ m!\.d/$model_file!
+                            and $n =~ m!\.d/$plugin_name!
                            ) ;
     } ;
-    find ($wanted, $snippet_dir ) ;
+    find ($wanted, $plugin_dir ) ;
 
     my $class_element = $self->meta_root->fetch_element('class') ;
 
     foreach my $load_file (@files) {
-        $logger->info("trying to read snippet $load_file");
+        $logger->info("trying to read plugin $load_file");
 
-        my $snippet = do $load_file ;
+        $load_file = "./$load_file" if $load_file !~ m!^/! and -e $load_file;
 
-        unless ($snippet) {
+        my $plugin = do $load_file ;
+
+        unless ($plugin) {
             if ($@) {die "couldn't parse $load_file: $@"; }
-            elsif (not defined $snippet) {die  "couldn't do $load_file: $!"}
+            elsif (not defined $plugin) {die  "couldn't do $load_file: $!"}
             else { die  "couldn't run $load_file" ;}
         }
 
-        # there should be only only class in each snippet file
-        foreach my $model (@$snippet) {
+        # there should be only only class in each plugin file
+        foreach my $model (@$plugin) {
             my $class_name = delete $model->{name} ;
             # load with a array ref to avoid warnings about missing order
             $class_element->fetch_with_id($class_name)->load_data( $model ) ;
         }
 
         # load annotations
-        $logger->info("loading annotations from snippet file $load_file");
+        $logger->info("loading annotations from plugin file $load_file");
         my $fh = IO::File->new($load_file) || die "Can't open $load_file: $!" ;
         my @lines = $fh->getlines ;
         $fh->close;
@@ -760,7 +799,7 @@ Config::Model::Itself - Model editor for Config::Model
 
 =head1 VERSION
 
-version 2.003
+version 2.013
 
 =head1 SYNOPSIS
 
@@ -791,12 +830,12 @@ Config::Itself module and its model files provide a model of Config:Model
 (hence the Itself name).
 
 Let's step back a little to explain. Any configuration data is, in
-essence, structured data. This data could be stored in an XML file. A
+essence, structured data. A
 configuration model is a way to describe the structure and relation of
 all items of a configuration data set.
 
 This configuration model is also expressed as structured data. This
-structure data is structured and follow a set of rules which are
+structure data follows a set of rules which are
 described for humans in L<Config::Model>.
 
 The structure and rules documented in L<Config::Model> are also
@@ -804,7 +843,7 @@ expressed in a model in the files provided with
 C<Config::Model::Itself>.
 
 Hence the possibity to verify, modify configuration data provided by
-Config::Model can also be applied on configuration models. Using the
+L<Config::Model> can also be applied on configuration models. Using the
 same user interface.
 
 From a Perl point of view, Config::Model::Itself provides a class
@@ -842,16 +881,17 @@ C<read_all> returns a hash ref containing ( class_name => file_name , ...)
 Will write back configuration model in the specified directory. The
 structure of the read directory is respected.
 
-=head2 write_model_snippet( snippet_dir => foo, model_file => bar.pl )
+=head2 write_model_plugin( plugin_dir => foo, plugin_name => bar )
 
-Write snippet models in separate C<.d> directory. E.g. a snippet for class
-C<Foo::Bar> will be written in C<Foo/Bar.d/bar.pl> file. This file is to be used
+Write plugin models in the  passed C<plugin_dir> directory. The written file is path is
+made of plugin name and class names. E.g. a plugin named C<bar> for class
+C<Foo::Bar> is written in C<bar/Foo/Bar.pl> file. This file is to be used
 by L<augment_config_class|Config::Model/"augment_config_class (name => '...', class_data )">
 
-=head2 read_model_snippet( snippet_dir => foo, model_file => bar.pl )
+=head2 read_model_plugin( plugin_dir => foo, plugin_name => bar.pl )
 
-To read model snippets, this methid will search recursively C<$snippet_dir> and load
-all C<bar.pl> files found in there.
+This method searched recursively C<$plugin_dir/$plugin_name> and load
+all C<*.pl> files found there.
 
 =head2 list_class_element
 
@@ -876,6 +916,13 @@ dashed lines. The name of the element is attached to the dashed line.
 
 =back
 
+=head1 BUGS
+
+Test menu entries are created from the content of C<application> model
+parameter.  Unfortunately, there's no way to build the menu
+dynamically. So user cme must be restarted to change the menu if the
+application list is changed.
+
 =head1 SEE ALSO
 
 L<Config::Model>, L<Config::Model::Node>, L<Path::Tiny>
@@ -886,10 +933,95 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2016 by Dominique Dumont.
+This software is Copyright (c) 2007-2017 by Dominique Dumont.
 
 This is free software, licensed under:
 
   The GNU Lesser General Public License, Version 2.1, February 1999
 
+=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
+
+=head1 SUPPORT
+
+=head2 Websites
+
+The following websites have more information about this module, and may be of help to you. As always,
+in addition to those websites please use your favorite search engine to discover more resources.
+
+=over 4
+
+=item *
+
+Search CPAN
+
+The default CPAN search engine, useful to view POD in HTML format.
+
+L<http://search.cpan.org/dist/Config-Model-Itself>
+
+=item *
+
+AnnoCPAN
+
+The AnnoCPAN is a website that allows community annotations of Perl module documentation.
+
+L<http://annocpan.org/dist/Config-Model-Itself>
+
+=item *
+
+CPAN Ratings
+
+The CPAN Ratings is a website that allows community ratings and reviews of Perl modules.
+
+L<http://cpanratings.perl.org/d/Config-Model-Itself>
+
+=item *
+
+CPANTS
+
+The CPANTS is a website that analyzes the Kwalitee ( code metrics ) of a distribution.
+
+L<http://cpants.cpanauthors.org/dist/Config-Model-Itself>
+
+=item *
+
+CPAN Testers
+
+The CPAN Testers is a network of smokers who run automated tests on uploaded CPAN distributions.
+
+L<http://www.cpantesters.org/distro/C/Config-Model-Itself>
+
+=item *
+
+CPAN Testers Matrix
+
+The CPAN Testers Matrix is a website that provides a visual overview of the test results for a distribution on various Perls/platforms.
+
+L<http://matrix.cpantesters.org/?dist=Config-Model-Itself>
+
+=item *
+
+CPAN Testers Dependencies
+
+The CPAN Testers Dependencies is a website that shows a chart of the test results of all dependencies for a distribution.
+
+L<http://deps.cpantesters.org/?module=Config::Model::Itself>
+
+=back
+
+=head2 Bugs / Feature Requests
+
+Please report any bugs or feature requests by email to C<ddumont at cpan.org>, or through
+the web interface at L<https://github.com/dod38fr/config-model-itself/issues>. You will be automatically notified of any
+progress on the request by the system.
+
+=head2 Source Code
+
+The code is open to the world, and available for you to hack on. Please feel free to browse it and play
+with it, or whatever. If you want to contribute patches, please send me a diff or prod me to pull
+from your repository :)
+
+L<http://github.com/dod38fr/config-model-itself>
+
+  git clone git://github.com/dod38fr/config-model-itself.git
+
 =cut
