@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2007-2016 by Dominique Dumont.
+# This software is Copyright (c) 2007-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -37,7 +37,7 @@
 
         'element' => [
 
-            [qw/class_description license/] => {
+            [qw/class_description license gist/] => {
                 type       => 'leaf',
                 value_type => 'string',
             },
@@ -100,8 +100,15 @@
                 type       => 'leaf',
                 value_type => 'uniline',
             },
+
+            rw_config => {
+                type              => 'node',
+                config_class_name => 'Itself::ConfigRead',
+            },
+
             'read_config' => {
                 type  => 'list',
+                status => 'deprecated',
                 cargo => {
                     type              => 'node',
                     config_class_name => 'Itself::ConfigRead',
@@ -110,6 +117,7 @@
 
             'write_config' => {
                 type  => 'list',
+                status => 'deprecated',
                 cargo => {
                     type              => 'node',
                     config_class_name => 'Itself::ConfigWrite',
@@ -127,27 +135,34 @@
         ],
         'description' => [
             element => "Specify the elements names of this configuration class.",
+            gist => 'String used to construct a summary of the content of a node. This '
+                .'parameter is used by user interface to show users the gist of the '
+                .'content of this node. This parameter has no other effect. This string '
+                .'may contain element values in the form "C<{foo} or {bar}>". When '
+                .'constructing the gist, C<{foo}> is replaced by the value of element '
+                .'C<foo>. Likewise for C<{bar}>.',
             include => "Include the element description of another class into this class.",
             include_after => "insert the included elements after a specific element. "
             . "By default, included elements are placed before all other elements.",
             include_backend => "Include the read/write specification of another class into this class.",
-            class_description => "Explain the purpose of this configuration class. This description will be re-used to generate the documentation of your configuration class. You can use pod markup to format your description. See L<perlpod> for details.",
-            read_config => "Specify the Perl class(es) and function(s) used to read configuration data. The specified function will be tried in sequence to get configuration data. ",
-            write_config => "Specify the Perl class and function used to write configuration data.",
+            class_description => "Explain the purpose of this configuration class. This description is re-used to generate the documentation of your configuration class. You can use pod markup to format your description. See L<perlpod> for details.",
+            rw_config => "Specify the backend used to read and write configuration data. See L<Config::Model::BackendMgr> for details",
+            read_config => "Obsolete - specify the Perl class(es) and function(s) used to read configuration data. The specified functions are tried in sequence to get configuration data. ",
+            write_config => "Obsolete - Specify the Perl class and function used to write configuration data.",
             generated_by => "When set, this class was generated by some program. You should not edit "
                 ."this class as your modifications may be clobbered later on when the class is regenerated.",
-            accept => "Specifies names of the elements this configuration class will accept as valid. "
-                ."The key of the hash is a regular expression that will be tested against 
-           candidate parameters. When the parameter matches the regular expression, 
+            accept => "Specifies names of the elements this configuration class accepts as valid. "
+                ."The key of the hash is a regular expression that are be tested against
+           candidate parameters. When the parameter matches the regular expression,
            a new parameter is created in the model using the description provided
            in the value of this hash key. Note that the regexp must match the whole name
-           of the potential parameter. I.e. the specified regexp will be eval\'ed 
+           of the potential parameter. I.e. the specified regexp is eval\'ed
            with a leading C<^> and a trailing C<\$>."
         ],
     ],
 
     [
-        name => 'Itself::ConfigWR::DefaultLayer',
+        name => 'Itself::ConfigReadWrite::DefaultLayer',
 
         'element'     => [
             'config_dir' => {
@@ -176,41 +191,37 @@
                 description => 'specify the configuration file name. This parameter may '
                     .'not be applicable depending on your application. It may also be '
                     .'hardcoded in a custom backend. If not specified, the instance name '
-                    .'will be used as base name for your configuration file.',
+                    .'is used as base name for your configuration file. The configuration file name'
+                    .'can be specified with &index keyword when a backend is associated to a node '
+                    .'contained in a hash. See '
+                    .'L<backend specifications|http://search.cpan.org/dist/Config-Model/lib/Config/Model/BackendMgr.pm#Backend_specification>.'
             },
         ]
     ],
 
     [
-        name => "Itself::ConfigWR",
-        include => "Itself::ConfigWR::DefaultLayer",
+        name => "Itself::ConfigReadWrite",
+        include => "Itself::ConfigReadWrite::DefaultLayer",
         include_after => 'backend',
 
         'element' => [
 
-            'syntax' => {
-                type       => 'leaf',
-                value_type => 'enum',
-                choice     => [qw/cds perl ini custom/],
-                status     => 'deprecated',
-                description => 'Deprecated parameter that specified the file syntax to store permanently configuration data. Replaced by "backend"',
-            },
-
             'backend' => {
                 type         => 'leaf',
                 class        => 'Config::Model::Itself::BackendDetector',
                 value_type   => 'enum',
                 choice       => [qw/cds_file perl_file custom/],
+                warn_if_match => {
+                    '^custom$' => {
+                        msg => "custom backend are deprecated"
+                    }
+                },
                 replace   => {
                     perl => 'perl_file',
                     ini  => 'IniFile',
                     ini_file  => 'IniFile',
                     cds  => 'cds_file',
                 },
-                migrate_from => {
-                    formula   => '$old',
-                    variables => { old => '- syntax' },
-                },
                 description => 'specifies the backend to store permanently configuration data.',
                 help => {
                     cds_file => "file with config data string. This is Config::Model own serialisation format, designed to be compact and readable. Configuration filename is made with instance name",
@@ -218,15 +229,45 @@
 "Ini file format. Beware that the structure of your model must match the limitations of the INI file format, i.e only a 2 levels hierarchy. Configuration filename is made with instance name",
                     perl_file =>
 "file with a perl data structure. Configuration filename is made with instance name",
-                    custom =>
-"Custom format. You must specify your own class and method to perform the read or write function. See Config::Model::AutoRead doc for more details",
+                    custom => "deprecated",
                }
             },
 
+            'auto_create' => {
+                type             => 'leaf',
+                value_type       => 'boolean',
+                level            => 'normal',
+                upstream_default => 0,
+                summary          => 'Creates configuration files as needed',
+            },
+
+            yaml_class => {
+                type             => 'leaf',
+                value_type       => 'uniline',
+                level            => 'hidden',
+                description      => 'Specify the YAML class that is used to load and dump YAML files.'
+                    .' Defaults to L<YAML::Tiny>.'
+                    .' See L<yaml_class doc|Config::Model::Backend::Yaml/yaml_class> for details on '
+                    .' why another YAML class can suit your configuration file needs.',
+                upstream_default => 'YAML::Tiny',
+                warp             => {
+                    follow => '- backend',
+                    rules  => [ Yaml => { level => 'normal', } ],
+                }
+            },
+
+            file_mode => {
+                type       => 'leaf',
+                value_type => 'uniline',
+                level      => 'normal',
+                summary     => 'configuration file mode',
+                description => 'specify the configuration file mode. C<file_mode> parameter can be used to set the '
+                    . 'mode of the written file. C<file_mode> value can be in any form supported by L<Path::Tiny/chmod>.'
+            },
 
             default_layer => {
                 type => 'node',
-                config_class_name => 'Itself::ConfigWR::DefaultLayer',
+                config_class_name => 'Itself::ConfigReadWrite::DefaultLayer',
                 summary => q!How to find default values in a global config file!,
                 description => q!Specifies where to find a global configuration file that !
                     .q!specifies default values. For instance, this is used by OpenSSH to !
@@ -253,7 +294,7 @@
                 type       => 'leaf',
                 value_type => 'uniline',
                 level      => 'hidden',
-                description => 'Specify element hash name that will contain all INI classes. '
+                description => 'Specify element hash name that contains all INI classes. '
                     .'See L<Config::Model::Backend::IniFile/"Arbitrary class name">',
                 warp => {
                     follow => '- backend',
@@ -265,13 +306,13 @@
                 type       => 'hash',
                 level      => 'hidden',
                 index_type => 'string',
-                description => 'Specify element name that will contain one INI class. E.g. to store '
+                description => 'Specify element name that contains one INI class. E.g. to store '
                      .'INI class [foo] in element Foo, specify { foo => "Foo" } ',
                 warp => {
                     follow => '- backend',
                     rules  => [ IniFile => { level => 'normal', } ],
                 },
-                cargo => { 
+                cargo => {
                     type => 'leaf',
                     value_type => 'uniline',
                 },
@@ -288,6 +329,32 @@
                 }
             },
 
+            assign_char => {
+                type       => 'leaf',
+                value_type => 'uniline',
+                level      => 'hidden',
+                description => 'Character used to assign value in INI file. Default is C<=>. '
+                    .'See L<details|Config::Model::Backend::IniFile/"Handle key value files">',
+                upstream_default => '#',
+                warp => {
+                    follow => '- backend',
+                    rules  => [ IniFile => { level => 'normal', } ],
+                }
+            },
+
+            assign_with => {
+                type       => 'leaf',
+                value_type => 'uniline',
+                level      => 'hidden',
+                description => 'String used write assignment in INI file. Default is "C< = >". '
+                    .'See L<details|Config::Model::Backend::IniFile/"Handle key value files">',
+                upstream_default => '#',
+                warp => {
+                    follow => '- backend',
+                    rules  => [ IniFile => { level => 'normal', } ],
+                }
+            },
+
             ['join_list_value', 'join_check_list_value'] => {
                 type       => 'leaf',
                 value_type => 'uniline',
@@ -303,7 +370,7 @@
                 description => 'Specify how to write a boolean value in config file. Suggested values are '
                     . '"no","yes". ',
                 max_index => 1,
-                cargo => { 
+                cargo => {
                     type => 'leaf',
                     value_type => 'uniline',
                 },
@@ -359,7 +426,10 @@
                 type             => 'leaf',
                 value_type       => 'uniline',
                 level            => 'hidden',
-                description      => 'comment starts with this character',
+                description      => 'list of characters that start a comment. When more that one character'
+                .' is used. the first one is used to write back comment. For instance,'
+                .' value "#;" indicate that a comments can start with "#" or ";" and that all comments'
+                .' are written back with "#".',
                 upstream_default => '#',
                 warp             => {
                     follow => '- backend',
@@ -388,7 +458,7 @@
 
     [
         name    => 'Itself::ConfigRead',
-        include => "Itself::ConfigWR",
+        include => "Itself::ConfigReadWrite",
 
         'element' => [
             'function' => {
@@ -405,34 +475,12 @@
                     ],
                 }
             },
-
-            'auto_create' => {
-                type             => 'leaf',
-                value_type       => 'boolean',
-                level            => 'normal',
-                upstream_default => 0,
-                summary          => 'Creates configuration files as needed',
-                migrate_from     => {
-                    formula   => '$old',
-                    variables => { old => '- allow_empty' },
-                },
-            },
-
-            'allow_empty' => {
-                type             => 'leaf',
-                value_type       => 'boolean',
-                level            => 'normal',
-                status           => 'deprecated',
-                upstream_default => 0,
-                summary          => 'deprecated in favor of auto_create',
-            },
-
         ],
     ],
 
     [
         name    => 'Itself::ConfigWrite',
-        include => "Itself::ConfigWR",
+        include => "Itself::ConfigReadWrite",
 
         'element' => [
             'function' => {
@@ -449,16 +497,6 @@
                     ],
                 }
             },
-
-            # move to ConfigRW when removing legacy allow_empty
-            'auto_create' => {
-                type             => 'leaf',
-                value_type       => 'boolean',
-                level            => 'normal',
-                upstream_default => 0,
-                summary          => 'Creates configuration files as needed',
-            },
-
         ],
     ],
     [
@@ -477,8 +515,8 @@
                 type => 'leaf',
                 value_type => 'reference' ,
                 refer_to => '- - element' ,
-                description => 'specify where to insert accepted element. This will'
-                 . ' not change the behavior but will help generating more consistent '
+                description => 'specify where to insert accepted element. This does'
+                 . ' not change the behavior and helps generating more consistent '
                  . ' user interfaces'
              }
 
