@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-Itself
 #
-# This software is Copyright (c) 2007-2016 by Dominique Dumont.
+# This software is Copyright (c) 2007-2017 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -85,8 +85,8 @@
                 },
                 description =>
                   "When set, a recurse copy of the value from the old object "
-                  . "to the new object will be attemped. When a copy is not possible, "
-                  . "undef values will be assigned.",
+                  . "to the new object is attemped. Old values are dropped when "
+                  ." a copy is not possible (usually because of mismatching types) "
             },
 
             # end warp elements for warped_node
@@ -135,7 +135,10 @@
                   . "elements in the configuration tree. The available choice "
                   . "of this reference value (or check list) is made from the "
                   . "available keys of the pointed hash element or the values "
-                  . "of the pointed array element.",
+                  . "of the pointed array element. The keys of several hashes (or lists) "
+                  . "can be combined by using the '+' operator in the formula. "
+                  . q(For instance, '! host:$a lan + ! host:foobar lan'. See )
+                  . "L<Config::Model::IdElementReference> for more details."
             },
 
             'replace_follow' => {
@@ -235,6 +238,19 @@
                     . 'in the configuration tree only when the hash is created.',
             },
 
+            write_empty_value => {
+                type       => 'leaf',
+                level      => 'hidden',
+                value_type => 'boolean',
+                upstream_default => 0,
+                warp       => {
+                    follow  => { 't'            => '?type' },
+                    rules   => [ '$t eq "hash"' => { level => 'normal', } ]
+                },
+                description => 'By default, hash entries without data are not saved in configuration '
+                    . 'files. Set this parameter to 1 if a key must be saved in the configuration '
+                    . 'file even if the hash contains no value for that key.',
+            },
             # list element
 
         ],
